﻿//
// Copyright (c) 2015, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using MindFusion.Gauges;
using System.Runtime.InteropServices;
using MindFusion;
using System.Drawing.Drawing2D;
using System.Runtime.CompilerServices;


namespace MindFusion.Gauges.WinForms.Samples.CS.ResourceMonitor
{
    public partial class MainForm : Form
    {
        PerformanceCounter pcProcessor;
        PerformanceCounter pcMemory;

        Pointer cpuPointer;
        Pointer ramPointer;
        OvalScale ramScale;
        OvalScale cpuScale;
        readonly int roundness = 25;

        Text cpuText;

        public MainForm()
        {
            InitializeComponent();

            pcProcessor = null;
            pcMemory = null;

            cpuPointer = cpuGauge.GetElementByName("CpuPointer") as Pointer;
            ramPointer = ramGauge.GetElementByName("RamPointer") as Pointer;
            cpuScale = cpuGauge.GetElementByName("CpuScale") as OvalScale;
            ramScale = ramGauge.GetElementByName("RamScale") as OvalScale;

            //set up ram gauge scale values
            ramScale.MinValue = 0;
            MemoryStatusEx stat = new MemoryStatusEx();
            GlobalMemoryStatusEx(stat);
            float ram = (float)stat.TotalPhysical;
            ramScale.MaxValue = ram/1024/1024;

            //CPU usage label
            cpuText = new Text();
            cpuText.FontFamily = "Consolas";
            cpuText.FontSize = new Length(20f, LengthType.Relative);
            cpuText.FontStyle = FontStyle.Bold;
            cpuText.Foreground = Color.FromArgb(0xFF, 0x7F, 0xFF, 0x0);
            cpuText.LineAlignment = StringAlignment.Center;
            cpuText.Alignment = StringAlignment.Center;
            cpuText.Content = "0%";
            cpuText.Margin = new Thickness(0, 0.3f, 0, 0);
            cpuScale.ScaleChildren.Add(cpuText);

            //Needed to make the back color transparent
            ramUsed.Parent = ramGauge;
            ramUsed.BackColor = Color.Transparent;
            ramFree.Parent = ramGauge;
            ramFree.BackColor = Color.Transparent;
            ramUsedMB.Parent = ramGauge;
            ramUsedMB.BackColor = Color.Transparent;

            //Add some event handlers
            cpuScale.QueryLabelValue += cpuScale_QueryLabelValue;
            ramScale.QueryLabelValue += ramScale_QueryLabelValue;
            cpuPointer.ValueChanged += cpuPointer_ValueChanged;
            cpuGauge.PrepaintForeground += cpuGauge_PrepaintForeground;
            cpuGauge.PrepaintBackground += cpuGauge_PrepaintBackground;
            ramGauge.PrepaintForeground += ramGauge_PrepaintForeground;
            ramGauge.PrepaintBackground += ramGauge_PrepaintBackground;
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            //Init performance counters and start the timer
            pcProcessor = new PerformanceCounter();
            pcProcessor.CategoryName = "Processor";
            pcProcessor.CounterName = "% Processor Time";
            pcProcessor.InstanceName = "_Total";

            pcMemory = new PerformanceCounter("Memory", "Available MBytes");

            timer.Enabled = true;
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            //Update values
            cpuPointer.Value = pcProcessor.NextValue();
            ramPointer.Value = pcMemory.NextValue();
            MemoryStatusEx stat = new MemoryStatusEx();
            GlobalMemoryStatusEx(stat);
            float percent = (float)stat.MemoryLoad;
            float free = stat.AvailablePhysical / 1024 / 1024;
            float used = (stat.TotalPhysical - stat.AvailablePhysical) / 1024 / 1024;
            ramUsed.Text = String.Format("RAM used: {0:0}%", percent);
            ramFree.Text = String.Format("Free: {0:0}MB", free);
            ramUsedMB.Text = String.Format("Used: {0:0}MB", used);
        }

        void ramGauge_PrepaintBackground(object sender, PrepaintEventArgs e)
        {
            e.CancelDefaultPainting = true;
            PaintCommonBackgroundElements(e);
        }

        void cpuGauge_PrepaintBackground(object sender, PrepaintEventArgs e)
        {
            e.CancelDefaultPainting = true;
            PaintCommonBackgroundElements(e);

            //CPU usage label background
            RoundRectangle textRect = new RoundRectangle();
            textRect.Fill = e.CreatePathGradient(
                new PointF(0.5f, 0.3f),
                0.0f, Color.FromArgb(0x80, 0xC0, 0xC0, 0xC0),
                0.2f, Color.FromArgb(0x9A, 0x64, 0x64, 0x64),
                0.6f, Color.FromArgb(0xBE, 0x0, 0x0, 0x0),
                1.0f, Color.FromArgb(0xFF, 0x0, 0x0, 0x0));
            MindFusion.Drawing.LinearGradientBrush strokeBrush =
                new MindFusion.Drawing.LinearGradientBrush(
                Color.FromArgb(0xFF, 0xA7, 0xA7, 0xA7),
                Color.FromArgb(0xFF, 0xFE, 0xFE, 0xFE),
                130f);
            textRect.Stroke = new MindFusion.Drawing.Pen(strokeBrush, 0.01f);
            textRect.Margin = new Thickness(0.37f, 0.55f, 0.37f, 0.32f);
            e.PaintVisualElement(textRect, e.Element.RenderSize);
        }

        void ramGauge_PrepaintForeground(object sender, PrepaintEventArgs e)
        {
            e.CancelDefaultPainting = true;
            PaintCommonForegroundElements(e);
        }

        void cpuGauge_PrepaintForeground(object sender, PrepaintEventArgs e)
        {
            e.CancelDefaultPainting = true;
            PaintCommonForegroundElements(e);
        }

        void cpuScale_QueryLabelValue(object sender, QueryLabelValueEventArgs e)
        {
            e.NewValue = String.Format("{0:0}%", e.CalculatedLabelValue);
        }

        void ramScale_QueryLabelValue(object sender, QueryLabelValueEventArgs e)
        {
            e.NewValue = String.Format("{0:0}MB", e.CalculatedLabelValue);
        }

        void cpuPointer_ValueChanged(object sender, ValueChangedEventArgs<float> e)
        {
            cpuText.Content = String.Format("{0:0}%", e.NewValue);
        }

        void PaintCommonBackgroundElements(PrepaintEventArgs e)
        {
            //Layer 1
            RoundRectangle rect = new RoundRectangle();
            rect.Margin = new Thickness(-0.11f, 0.2f, -0.11f, 0.08f);
            rect.Fill = e.CreateLinearGradient(45f,
                0.0f, Color.FromArgb(0xFF, 0x32, 0x32, 0x32),
                0.2f, Color.FromArgb(0xFF, 0x32, 0x32, 0x32),
                0.8f, Color.FromArgb(0xFF, 0x64, 0x64, 0x64),
                1.0f, Color.FromArgb(0xFF, 0x64, 0x64, 0x64));
            rect.Roundness = roundness;
            e.PaintVisualElement(rect, e.Element.RenderSize);

            //Layer 2
            rect.Margin = new Thickness(-0.1f, 0.21f, -0.1f, 0.09f);
            rect.Fill = e.CreateLinearGradient(45f,
                0.0f, Color.FromArgb(0xFF, 0x64, 0x64, 0x64),
                0.2f, Color.FromArgb(0xFF, 0x64, 0x64, 0x64),
                0.8f, Color.FromArgb(0xFF, 0x32, 0x32, 0x32),
                1.0f, Color.FromArgb(0xFF, 0x32, 0x32, 0x32));
            rect.Roundness = roundness;
            e.PaintVisualElement(rect, e.Element.RenderSize);

            //Layer 3
            rect.Margin = new Thickness(-0.09f, 0.22f, -0.09f, 0.1f);
            rect.Fill = e.CreateLinearGradient(45f,
                0.0f, Color.FromArgb(0xFF, 0x32, 0x32, 0x32),
                0.2f, Color.FromArgb(0xFF, 0x32, 0x32, 0x32),
                0.8f, Color.FromArgb(0xFF, 0x64, 0x64, 0x64),
                1.0f, Color.FromArgb(0xFF, 0x64, 0x64, 0x64));
            rect.Roundness = roundness;
            e.PaintVisualElement(rect, e.Element.RenderSize);
        }

        void PaintCommonForegroundElements(PrepaintEventArgs e)
        {
            //Layer 1
            RoundRectangle rect = new RoundRectangle();
            rect.Margin = new Thickness(-0.09f, 0.22f, -0.09f, 0.1f);
            rect.Fill = e.CreateLinearGradient(45f,
                0.0f, Color.FromArgb(0x64, 0xFF, 0xFF, 0xFF),
                0.2f, Color.FromArgb(0x32, 0xFF, 0xFF, 0xFF),
                0.8f, Color.FromArgb(0x10, 0xFF, 0xFF, 0xFF),
                1.0f, Color.FromArgb(0x0, 0xFF, 0xFF, 0xFF));
            rect.Roundness = roundness;
            //rect.Stroke = new MindFusion.Drawing.Pen((Color.Transparent), 0);
            e.PaintVisualElement(rect, e.Element.RenderSize);

            RectangleF bounds = new RectangleF(0, 0, 1, 1);
            bounds.Inflate(0, -0.2f);

            //Reflections
            GraphicsPath areaPath = new GraphicsPath();
            areaPath.AddEllipse(bounds);

            MindFusion.Drawing.PathGradientBrush area1Fill = new MindFusion.Drawing.PathGradientBrush(Color.FromArgb(153, Color.White), Color.Transparent);
            area1Fill.Path = areaPath;

            MindFusion.Drawing.PathGradientBrush area2Fill = new MindFusion.Drawing.PathGradientBrush(Color.FromArgb(32, Color.White), Color.Transparent);
            area2Fill.Path = areaPath;

            ArcArea area1 = new ArcArea();
            area1.StartAngle = 160;
            area1.EndAngle = -20;
            area1.Margin = new Thickness(0.03f);
            area1.Fill = area1Fill;

            ArcArea area2 = new ArcArea();
            area2.StartAngle = 140;
            area2.EndAngle = -40;
            area2.Margin = new Thickness(0.03f);
            area2.Fill = area2Fill;

            e.PaintVisualElement(area1, e.Element.RenderSize);
            e.PaintVisualElement(area2, e.Element.RenderSize);

            areaPath.Dispose();
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
        private class MemoryStatusEx
        {
            public uint Length;
            public uint MemoryLoad;
            public ulong TotalPhysical;
            public ulong AvailablePhysical;
            public ulong TotalPageFile;
            public ulong AvailablePageFile;
            public ulong TotalVirtual;
            public ulong AvailableVirtual;
            public ulong AvailableExtendedVirtual;
            public MemoryStatusEx()
            {
                this.Length = (uint)Marshal.SizeOf(this);
            }
        }

        [return: MarshalAs(UnmanagedType.Bool)]
        [DllImport("kernel32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        static extern bool GlobalMemoryStatusEx([In, Out] MemoryStatusEx lpBuffer);
    }
}